@@ -3,10 +3,13 @@
 import os
 import unittest
 
+
 from xml.dom import minidom
 
+
 import epub
 
+
 class TestFunction(unittest.TestCase):
     """Test case for all the _parse_* function."""
 
@@ -17,7 +20,7 @@ class TestFunction(unittest.TestCase):
         doc = minidom.parse(test_path)
         xml_string = doc.toprettyxml()
         toc = epub.ncx.parse_toc(xml_string)
-        
+
         self.assertEqual(toc.xmlns,
                          u'http://www.daisy.org/z3986/2005/ncx/')
         self.assertEqual(toc.version,
@@ -85,7 +88,7 @@ class TestFunction(unittest.TestCase):
         xml_element = minidom.parseString(xml_string.encode(u'utf-8')).documentElement
         text = epub.ncx._parse_for_text_tag(xml_element)
         self.assertEqual(text, u'Première balise uniquement...')
-        
+
         xml_string = u"""
         <someTag>
             <text>Je ne cherche pas "text" !</text>
@@ -107,9 +110,18 @@ class TestFunction(unittest.TestCase):
         self.assertEqual(text, u'Du texte avec espace et retour à la ligne.',
                          u'Il ne devrait pas y avoir de différence !')
 
+        xml_string = u"""
+        <someTag>
+            <emptyText />
+        </someTag>
+        """
+        xml_element = minidom.parseString(xml_string.encode(u'utf-8')).documentElement
+        text = epub.ncx._parse_for_text_tag(xml_element, u'emptyText')
+        self.assertEqual(text, u'')
+
     def test_parse_xml_nav_target(self):
         """Test function "_parse_xml_nav_target"."""
-        
+
         xml_string = u"""
         <navTarget playOrder="5"
                    id="part1_target-fragment"
@@ -122,13 +134,13 @@ class TestFunction(unittest.TestCase):
         xml_element = minidom.parseString(xml_string.encode(u'utf-8')).documentElement
         nav_target = epub.ncx._parse_xml_nav_target(xml_element)
         self.assertIsInstance(nav_target, epub.ncx.NavTarget)
-        self.assertEqual(nav_target.id, u'part1_target-fragment')
+        self.assertEqual(nav_target.identifier, u'part1_target-fragment')
         self.assertEqual(nav_target.value, u'5')
         self.assertEqual(nav_target.class_name, u'some_class')
         self.assertEqual(nav_target.play_order, u'5')
         self.assertEqual(nav_target.labels, [(u'Label de la Target', u'', u''),])
         self.assertEqual(nav_target.src, u'Text/part1.xhtml#target-fragment')
-        
+
         xml_string = u"""
         <navTarget>
             <navLabel xml:lang="fr"><text>Label français</text></navLabel>
@@ -143,7 +155,7 @@ class TestFunction(unittest.TestCase):
 
     def test_parse_xml_nav_list(self):
         """Test function "_parse_xml_nav_list"."""
-        
+
         xml_string = u"""
         <navList id="navlist-1" class="list_class">
             <navLabel xml:lang="fr"><text>Label français</text></navLabel>
@@ -169,7 +181,7 @@ class TestFunction(unittest.TestCase):
         xml_element = minidom.parseString(xml_string.encode(u'utf-8')).documentElement
         nav_list = epub.ncx._parse_xml_nav_list(xml_element)
         self.assertIsInstance(nav_list, epub.ncx.NavList)
-        self.assertEqual(nav_list.id, u'navlist-1')
+        self.assertEqual(nav_list.identifier, u'navlist-1')
         self.assertEqual(nav_list.class_name, u'list_class')
         self.assertEqual(nav_list.labels, [(u'Label français', u'fr', u''),
                                              (u'English label', u'en', u''),])
@@ -185,7 +197,7 @@ class TestFunction(unittest.TestCase):
 
     def test_parse_xml_page_target(self):
         """Test function "_parse_xml_page_target"."""
-        
+
         xml_string = u"""
         <pageTarget playOrder="5"
                    id="part1_target-fragment"
@@ -199,14 +211,14 @@ class TestFunction(unittest.TestCase):
         xml_element = minidom.parseString(xml_string.encode(u'utf-8')).documentElement
         page_target = epub.ncx._parse_xml_page_target(xml_element)
         self.assertIsInstance(page_target, epub.ncx.PageTarget)
-        self.assertEqual(page_target.id, u'part1_target-fragment')
+        self.assertEqual(page_target.identifier, u'part1_target-fragment')
         self.assertEqual(page_target.value, u'Some Value')
-        self.assertEqual(page_target.type, u'page_type')
+        self.assertEqual(page_target.target_type, u'page_type')
         self.assertEqual(page_target.class_name, u'some_class')
         self.assertEqual(page_target.play_order, u'5')
         self.assertEqual(page_target.labels, [(u'Label de la Target', u'', u''),])
         self.assertEqual(page_target.src, u'Text/part1.xhtml#target-fragment')
-        
+
         xml_string = u"""
         <pageTarget>
             <navLabel xml:lang="fr"><text>Label français</text></navLabel>
@@ -250,7 +262,7 @@ class TestFunction(unittest.TestCase):
         xml_element = minidom.parseString(xml_string.encode(u'utf-8')).documentElement
         page_list = epub.ncx._parse_xml_page_list(xml_element)
         self.assertIsInstance(page_list, epub.ncx.PageList)
-        self.assertEqual(page_list.id, u'pagelist-1')
+        self.assertEqual(page_list.identifier, u'pagelist-1')
         self.assertEqual(page_list.class_name, u'page_class')
         self.assertEqual(page_list.labels, [(u'Label français', u'fr', u''),
                                              (u'English label', u'en', u''),])
@@ -287,7 +299,7 @@ class TestFunction(unittest.TestCase):
         xml_element = minidom.parseString(xml_string.encode(u'utf-8')).documentElement
         nav_point = epub.ncx._parse_xml_nav_point(xml_element)
         self.assertIsInstance(nav_point, epub.ncx.NavPoint)
-        self.assertEqual(nav_point.id, u'point5')
+        self.assertEqual(nav_point.identifier, u'point5')
         self.assertEqual(nav_point.class_name, u'some_class')
         self.assertEqual(nav_point.labels, [(u'Label fr', 'fr', ''),
                                              (u'Label en', 'en', ''),])
@@ -326,20 +338,28 @@ class TestFunction(unittest.TestCase):
         xml_element = minidom.parseString(xml_string.encode(u'utf-8')).documentElement
         nav_map = epub.ncx._parse_xml_nav_map(xml_element)
         self.assertIsInstance(nav_map, epub.ncx.NavMap)
-        self.assertEqual(nav_map.id, u'the_only_one_map')
+        self.assertEqual(nav_map.identifier, u'the_only_one_map')
         self.assertEqual(len(nav_map.nav_point), 2)
         self.assertEqual(len(nav_map.nav_point[0].nav_point), 0)
         self.assertEqual(len(nav_map.nav_point[1].nav_point), 2)
 
     def test_create_xml_element_text(self):
-        xml_string = """<text>Some text</text>"""
+        xml_string = u"""<text>Some text</text>"""
         xml_element = epub.ncx._create_xml_element_text(u'Some text')
         self.assertEqual(xml_element.toxml(), xml_string)
         
-        xml_string = """<otherText>Some text</otherText>"""
+        xml_string = u"""<otherText>Some text</otherText>"""
         xml_element = epub.ncx._create_xml_element_text(u'Some text', u'otherText')
         self.assertEqual(xml_element.toxml(), xml_string)
 
+        xml_string = u"""<emptyText/>"""
+        xml_element = epub.ncx._create_xml_element_text(None, u'emptyText')
+        self.assertEqual(xml_element.toxml(), xml_string)
+
+        xml_string = u"""<emptyText/>"""
+        xml_element = epub.ncx._create_xml_element_text(u'', u'emptyText')
+        self.assertEqual(xml_element.toxml(), xml_string)
+
 
 class TestNavPoint(unittest.TestCase):
     
@@ -368,7 +388,7 @@ class TestNavPoint(unittest.TestCase):
 class TestNavMap(unittest.TestCase):
     
     def test_as_xml_element(self):
-        xml_string = u"""<navMap>""" + \
+        xml_string = u"""<navMap id="someId">""" + \
             """<navLabel xml:lang="fr" dir="ltr"><text>Label fr</text></navLabel>""" + \
             """<navLabel xml:lang="en" dir="ltr"><text>Label en</text></navLabel>""" + \
             """<navInfo xml:lang="fr" dir="ltr"><text>Label fr</text></navInfo>""" + \
@@ -479,42 +499,30 @@ class TestNcx(unittest.TestCase):
         <meta content="Epub Test" name="dtb:generator"/>
     </head>
     <docTitle>
-        <text>
-            Selections from &quot;Great Pictures, As Seen and Described by Famous Writers&quot;
-        </text>
+        <text>Selections from &quot;Great Pictures, As Seen and Described by Famous Writers&quot;</text>
     </docTitle>
     <docAuthor>
-        <text>
-            Esther Singleton
-        </text>
+        <text>Esther Singleton</text>
     </docAuthor>
     <docAuthor>
-        <text>
-            Test Author
-        </text>
+        <text>Test Author</text>
     </docAuthor>
     <navMap>
         <navPoint class="h1" id="ch1">
             <navLabel>
-                <text>
-                    Chapter 1
-                </text>
+                <text>Chapter 1</text>
             </navLabel>
             <content src="content.html#ch_1"/>
             <navPoint class="h2" id="ch_1_1">
                 <navLabel>
-                    <text>
-                        Chapter 1.1
-                    </text>
+                    <text>Chapter 1.1</text>
                 </navLabel>
                 <content src="content.html#ch_1_1"/>
             </navPoint>
         </navPoint>
         <navPoint class="h1" id="ncx-2">
             <navLabel>
-                <text>
-                    Chapter 2
-                </text>
+                <text>Chapter 2</text>
             </navLabel>
             <content src="content.html#ch_2"/>
         </navPoint>
@@ -522,40 +530,30 @@ class TestNcx(unittest.TestCase):
     <pageList>
         <pageTarget id="p1" type="normal" value="1">
             <navLabel dir="ltr">
-                <text>
-                    1
-                </text>
+                <text>1</text>
             </navLabel>
             <content src="content.html#p1"/>
         </pageTarget>
         <pageTarget id="p2" type="normal" value="2">
             <navLabel>
-                <text>
-                    2
-                </text>
+                <text>2</text>
             </navLabel>
             <content src="content.html#p2"/>
         </pageTarget>
     </pageList>
     <navList>
         <navLabel>
-            <text>
-                List of Illustrations
-            </text>
+            <text>List of Illustrations</text>
         </navLabel>
         <navTarget id="ill-1">
             <navLabel>
-                <text>
-                    Portratit of Georg Gisze (Holbein)
-                </text>
+                <text>Portratit of Georg Gisze (Holbein)</text>
             </navLabel>
             <content src="content.html#ill1"/>
         </navTarget>
         <navTarget id="ill-2">
             <navLabel>
-                <text>
-                    The adoration of the lamb (Van Eyck)
-                </text>
+                <text>The adoration of the lamb (Van Eyck)</text>
             </navLabel>
             <content src="content.html#ill2"/>
         </navTarget>
@@ -564,15 +562,11 @@ class TestNcx(unittest.TestCase):
 """
 
         self.maxDiff = None
-        
+
         toc = epub.ncx.parse_toc(xml_string)
         xml_toc = toc.as_xml_document().toprettyxml(u'    ')
-        
+
         xml_input = xml_string.strip()
         xml_output = xml_toc.strip()
-        
-        print xml_input
-        print xml_output
 
         self.assertEqual(xml_output, xml_input)
-
