@@ -9,6 +9,9 @@ import epub
 
 class TestFunction(unittest.TestCase):
 
+    def test_version(self):
+        self.assertEqual(epub.__version__, u'0.4.0')
+
     def test_parse_opf(self):
         xml_string = u"""<?xml version="1.0" ?>
 <package unique-identifier="BookId" version="2.0" xmlns="http://www.idpf.org/2007/opf">
@@ -54,7 +57,7 @@ class TestFunction(unittest.TestCase):
 
     def test_parse_xml_metadata(self):
         """Test _parse_xml_metadata."""
-        
+
         xml_string = u"""
         <metadata xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:opf="http://www.idpf.org/2007/opf">
             <dc:identifier opf:scheme="epub_test" id="epub_id">
@@ -126,68 +129,70 @@ class TestFunction(unittest.TestCase):
         """
         element = minidom.parseString(xml_string.encode(u'utf-8')).documentElement
         metadata = epub.opf._parse_xml_metadata(element)
-        
+
         # dc:identifier
         self.assertEqual(metadata.identifiers,
                          [(u'1', u'epub_id', u'epub_test'),
                           (u'18430f16-c687-400b-9b9c-54a9c8b0646c', u'uuid_id', u'uuid')])
-        
+
         # dc:title
         self.assertEqual(metadata.titles,
                          [(u'Metadata for testing purpose', ''),
                           (u'Metadonnée pour les tests.', u'fr')])
-        
+
         # dc:creator
         self.assertEqual(metadata.creators,
-                         [(u'John Doe', u'aut', u'Doe, Jhon'),])
-        
+                         [(u'John Doe', u'aut', u'Doe, Jhon'), ])
+
         # dc:subject
         self.assertEqual(metadata.subjects,
                          [u'This is an arbitrary subjet.',
                           u'Another usefull subject.'])
-        
+
         # dc:description
         self.assertEqual(metadata.description,
-                         u"""A long description. There is not any information about how a 
+            u"""A long description. There is not any information about how a 
                 description must be. Long, short, etc.
                 
                 We just don't know anything about this.""")
-        
+
         # dc:contributor
         self.assertEqual(metadata.contributors,
-                         [(u'Python unittest', u'other.test', u'Python, unittest'),
-                          (u'Python nosetests', u'other.test', u'Python, nosetests')])
-        
+                         [(u'Python unittest', u'other.test',
+                           u'Python, unittest'),
+                          (u'Python nosetests', u'other.test',
+                           u'Python, nosetests')])
+
         # dc:date
         self.assertEqual(metadata.dates,
                          [(u'2012-01-05T16:18:00+00:00', u'creation'),
                           (u'2012-01-09T13:37:00+00:00', u'publication')])
-        
+
         # dc:type
-        self.assertEqual(metadata.type, u'Is this a type?')
-        
+        self.assertEqual(metadata.dc_type, u'Is this a type?')
+
         # dc:format
         self.assertEqual(metadata.format, u'Well formated, sir!')
-        
+
         # dc:publisher
         self.assertEqual(metadata.publisher, u'Exirel')
-        
+
         # dc:language
         self.assertEqual(metadata.languages,
-                         [u'en',])
-        
+                         [u'en'])
+
         # dc:source
         self.assertEqual(metadata.source,
                          u'From the far old west (Brittany, France).')
-        
+
         # dc:relation
         self.assertEqual(metadata.relation,
                          u'It\'s complicated...')
-        
+
         # dc:coverage
         self.assertEqual(metadata.coverage,
                          u'An art of cover.')
-        
+
         # dc:rights
         self.assertEqual(metadata.right,
                          u'To the left!')
@@ -211,15 +216,16 @@ class TestFunction(unittest.TestCase):
         self.assertIsInstance(manifest, epub.opf.Manifest,
                               u'Manifest must be an epub.opf.Manifest instance.')
         self.assertEqual(len(manifest), 4)
-        
-        for key, item in manifest.iteritems():
+
+        for item in manifest.values():
+            print item
             self.assertIsInstance(item, epub.opf.ManifestItem)
-        
-        self.assertEqual(manifest[u'toc'].id, u'toc')
-        self.assertEqual(manifest[u'cover'].id, u'cover')
-        self.assertEqual(manifest[u'chap1'].id, u'chap1')
-        self.assertEqual(manifest[u'chap2'].id, u'chap2')
-        
+
+        self.assertEqual(manifest[u'toc'].identifier, u'toc')
+        self.assertEqual(manifest[u'cover'].identifier, u'cover')
+        self.assertEqual(manifest[u'chap1'].identifier, u'chap1')
+        self.assertEqual(manifest[u'chap2'].identifier, u'chap2')
+
         self.assertEqual(manifest[u'toc'].href, u'toc.ncx')
         self.assertEqual(manifest[u'cover'].href, u'Text/cover.html')
         self.assertEqual(manifest[u'chap1'].href, u'Text/chap1.html')
@@ -227,47 +233,231 @@ class TestFunction(unittest.TestCase):
 
 
 class TestMetadata(unittest.TestCase):
-    
+
     def test_add_identifier(self):
         metadata = epub.opf.Metadata()
         self.assertEqual(metadata.identifiers, [])
-        
+
         metadata.add_identifier(u'978-284172074-3', u'ID_ISBN', u'isbn')
-        
+
         self.assertEqual(metadata.identifiers,
                          [(u'978-284172074-3', u'ID_ISBN', u'isbn')])
-        
+
         metadata.add_identifier(u'781445645135453123543', u'ID_UID', u'UID')
-        
+
         self.assertEqual(metadata.identifiers,
                          [(u'978-284172074-3', u'ID_ISBN', u'isbn'),
                           (u'781445645135453123543', u'ID_UID', u'UID')])
-        
+
     def test_get_isbn(self):
         metadata = epub.opf.Metadata()
         self.assertEqual(metadata.get_isbn(), None)
-        
+
         metadata.add_identifier(u'978-284172074-3', u'ID_ISBN', u'isbn')
         metadata.add_identifier(u'781445645135453123543', u'ID_UID', u'UID')
-        
+
         self.assertEqual(metadata.get_isbn(), u'978-284172074-3')
 
     def test_as_xml_element(self):
         xml_string = u"""<metadata xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:opf="http://www.idpf.org/2007/opf">
-    <dc:title xml:lang="fr">
-        Le titre.
-    </dc:title>
-    <dc:title xml:lang="en">
-        The title.
-    </dc:title>
+    <dc:title xml:lang="fr">Le titre.</dc:title>
+    <dc:title xml:lang="en">The title.</dc:title>
 </metadata>"""
 
         self.maxDiff = None
         doc = minidom.parseString(xml_string)
         xml_element = doc.documentElement
-        
+
         metadata = epub.opf._parse_xml_metadata(xml_element)
-        
+
+        self.assertEqual(metadata.as_xml_element().toprettyxml(u'    ').strip(),
+                         xml_element.toxml().strip())
+
+    def test_as_xml_element_creators(self):
+        xml_string = u"""<metadata xmlns:dc="%s" xmlns:opf="%s">
+    <dc:creator>First Creator of the Book</dc:creator>
+    <dc:creator opf:file-as="aut">Second Creator with File-as attribute</dc:creator>
+    <dc:creator opf:role="translator">Third Creator with role</dc:creator>
+    <dc:creator opf:role="translator" opf:file-as="aut">Last creator with All</dc:creator>
+</metadata>""" % (epub.opf.XMLNS_DC, epub.opf.XMLNS_OPF)
+
+        self.maxDiff = None
+        doc = minidom.parseString(xml_string)
+        xml_element = doc.documentElement
+        xml_element.normalize()
+
+        metadata = epub.opf._parse_xml_metadata(xml_element)
+
+        self.assertEqual(metadata.as_xml_element().toprettyxml(u'    ').strip(),
+                         xml_element.toxml().strip())
+
+    def test_as_xml_element_subjects(self):
+        xml_string = u"""<metadata xmlns:dc="%s" xmlns:opf="%s">
+    <dc:subject>Subject One</dc:subject>
+    <dc:subject>Subject Two</dc:subject>
+</metadata>""" % (epub.opf.XMLNS_DC, epub.opf.XMLNS_OPF)
+
+        self.maxDiff = None
+        doc = minidom.parseString(xml_string)
+        xml_element = doc.documentElement
+        xml_element.normalize()
+
+        metadata = epub.opf._parse_xml_metadata(xml_element)
+
+        self.assertEqual(metadata.as_xml_element().toprettyxml(u'    ').strip(),
+                         xml_element.toxml().strip())
+
+    def test_as_xml_element_description(self):
+        xml_string = u"""<metadata xmlns:dc="%s" xmlns:opf="%s">
+    <dc:description>This is a very long description. I don't know how you could use it, but...</dc:description>
+</metadata>""" % (epub.opf.XMLNS_DC, epub.opf.XMLNS_OPF)
+
+        self.maxDiff = None
+        doc = minidom.parseString(xml_string)
+        xml_element = doc.documentElement
+        xml_element.normalize()
+
+        metadata = epub.opf._parse_xml_metadata(xml_element)
+
+        self.assertEqual(metadata.as_xml_element().toprettyxml(u'    ').strip(),
+                         xml_element.toxml().strip())
+
+    def test_as_xml_element_publisher(self):
+        xml_string = u"""<metadata xmlns:dc="%s" xmlns:opf="%s">
+    <dc:publisher>Exirel Python Epub</dc:publisher>
+</metadata>""" % (epub.opf.XMLNS_DC, epub.opf.XMLNS_OPF)
+
+        self.maxDiff = None
+        doc = minidom.parseString(xml_string)
+        xml_element = doc.documentElement
+        xml_element.normalize()
+
+        metadata = epub.opf._parse_xml_metadata(xml_element)
+
+        self.assertEqual(metadata.as_xml_element().toprettyxml(u'    ').strip(),
+                         xml_element.toxml().strip())
+
+    def test_as_xml_element_contributors(self):
+        xml_string = u"""<metadata xmlns:dc="%s" xmlns:opf="%s">
+    <dc:contributor>First Contributor of the Book</dc:contributor>
+    <dc:contributor opf:file-as="aut">Second Contributor with File-as attribute</dc:contributor>
+    <dc:contributor opf:role="translator">Third Contributor with role</dc:contributor>
+    <dc:contributor opf:role="translator" opf:file-as="aut">Last Contributor with All</dc:contributor>
+</metadata>""" % (epub.opf.XMLNS_DC, epub.opf.XMLNS_OPF)
+
+        self.maxDiff = None
+        doc = minidom.parseString(xml_string)
+        xml_element = doc.documentElement
+        xml_element.normalize()
+
+        metadata = epub.opf._parse_xml_metadata(xml_element)
+
+        self.assertEqual(metadata.as_xml_element().toprettyxml(u'    ').strip(),
+                         xml_element.toxml().strip())
+
+    def test_as_xml_element_date(self):
+        xml_string = u"""<metadata xmlns:dc="%s" xmlns:opf="%s">
+    <dc:date opf:event="php_end">2008-08-08</dc:date>
+    <dc:date>2009-09-09</dc:date>
+</metadata>""" % (epub.opf.XMLNS_DC, epub.opf.XMLNS_OPF)
+
+        self.maxDiff = None
+        doc = minidom.parseString(xml_string)
+        xml_element = doc.documentElement
+        xml_element.normalize()
+
+        metadata = epub.opf._parse_xml_metadata(xml_element)
+
+        self.assertEqual(metadata.as_xml_element().toprettyxml(u'    ').strip(),
+                         xml_element.toxml().strip())
+
+    def test_as_xml_element_type(self):
+        xml_string = u"""<metadata xmlns:dc="%s" xmlns:opf="%s">
+    <dc:type>Epub Type</dc:type>
+</metadata>""" % (epub.opf.XMLNS_DC, epub.opf.XMLNS_OPF)
+
+        self.maxDiff = None
+        doc = minidom.parseString(xml_string)
+        xml_element = doc.documentElement
+        xml_element.normalize()
+
+        metadata = epub.opf._parse_xml_metadata(xml_element)
+
+        self.assertEqual(metadata.as_xml_element().toprettyxml(u'    ').strip(),
+                         xml_element.toxml().strip())
+
+    def test_as_xml_element_format(self):
+        xml_string = u"""<metadata xmlns:dc="%s" xmlns:opf="%s">
+    <dc:format>Epub Format</dc:format>
+</metadata>""" % (epub.opf.XMLNS_DC, epub.opf.XMLNS_OPF)
+
+        self.maxDiff = None
+        doc = minidom.parseString(xml_string)
+        xml_element = doc.documentElement
+        xml_element.normalize()
+
+        metadata = epub.opf._parse_xml_metadata(xml_element)
+
+        self.assertEqual(metadata.as_xml_element().toprettyxml(u'    ').strip(),
+                         xml_element.toxml().strip())
+
+    def test_as_xml_element_source(self):
+        xml_string = u"""<metadata xmlns:dc="%s" xmlns:opf="%s">
+    <dc:source>Epub Format</dc:source>
+</metadata>""" % (epub.opf.XMLNS_DC, epub.opf.XMLNS_OPF)
+
+        self.maxDiff = None
+        doc = minidom.parseString(xml_string)
+        xml_element = doc.documentElement
+        xml_element.normalize()
+
+        metadata = epub.opf._parse_xml_metadata(xml_element)
+
+        self.assertEqual(metadata.as_xml_element().toprettyxml(u'    ').strip(),
+                         xml_element.toxml().strip())
+
+    def test_as_xml_element_relation(self):
+        xml_string = u"""<metadata xmlns:dc="%s" xmlns:opf="%s">
+    <dc:relation>Epub Relation</dc:relation>
+</metadata>""" % (epub.opf.XMLNS_DC, epub.opf.XMLNS_OPF)
+
+        self.maxDiff = None
+        doc = minidom.parseString(xml_string)
+        xml_element = doc.documentElement
+        xml_element.normalize()
+
+        metadata = epub.opf._parse_xml_metadata(xml_element)
+
+        self.assertEqual(metadata.as_xml_element().toprettyxml(u'    ').strip(),
+                         xml_element.toxml().strip())
+
+    def test_as_xml_element_coverage(self):
+        xml_string = u"""<metadata xmlns:dc="%s" xmlns:opf="%s">
+    <dc:coverage>Epub Coverage</dc:coverage>
+</metadata>""" % (epub.opf.XMLNS_DC, epub.opf.XMLNS_OPF)
+
+        self.maxDiff = None
+        doc = minidom.parseString(xml_string)
+        xml_element = doc.documentElement
+        xml_element.normalize()
+
+        metadata = epub.opf._parse_xml_metadata(xml_element)
+
+        self.assertEqual(metadata.as_xml_element().toprettyxml(u'    ').strip(),
+                         xml_element.toxml().strip())
+
+    def test_as_xml_element_right(self):
+        xml_string = u"""<metadata xmlns:dc="%s" xmlns:opf="%s">
+    <dc:rights>Epub Rights</dc:rights>
+</metadata>""" % (epub.opf.XMLNS_DC, epub.opf.XMLNS_OPF)
+
+        self.maxDiff = None
+        doc = minidom.parseString(xml_string)
+        xml_element = doc.documentElement
+        xml_element.normalize()
+
+        metadata = epub.opf._parse_xml_metadata(xml_element)
+
         self.assertEqual(metadata.as_xml_element().toprettyxml(u'    ').strip(),
                          xml_element.toxml().strip())
 
@@ -275,28 +465,28 @@ class TestMetadata(unittest.TestCase):
 class TestManifest(unittest.TestCase):
 
     class TestDuckManifestItem(object):
-        
-        def __init__(self, id=None, href=None):
-            self.id = id
+
+        def __init__(self, identifier=None, href=None):
+            self.identifier = identifier
             if href:
                 self.href = href
 
     def test_dict_behavior(self):
-        id = u'ID001'
+        identifier = u'ID001'
         bad_id = u'BAD_ID001'
         duck_id = u'FAKE_ID001'
         duck_href = u'fake_item_001.xhtml'
         href = u'item_001.xhtml'
         media = u'application/xhtml+xml'
         manifest = epub.opf.Manifest()
-        manifest_item = epub.opf.ManifestItem(id, href, media)
-        
+        manifest_item = epub.opf.ManifestItem(identifier, href, media)
+
         self.assertEqual(len(manifest), 0)
-        
-        manifest[id] = manifest_item
-        
+
+        manifest[identifier] = manifest_item
+
         self.assertEqual(len(manifest), 1)
-        self.assertEqual(manifest[id], manifest_item)
+        self.assertEqual(manifest[identifier], manifest_item)
 
         with self.assertRaises(ValueError):
             manifest[bad_id] = manifest_item
@@ -312,24 +502,24 @@ class TestManifest(unittest.TestCase):
             duck_ko = self.TestDuckManifestItem(duck_id)
             manifest.append(duck_ko)
 
-        self.assertTrue(id in manifest)
+        self.assertTrue(identifier in manifest)
         self.assertTrue(manifest_item in manifest)
         self.assertTrue(duck_ok in manifest)
 
     def test_add_item(self):
         """Check epub.opf.Manifest.add_item()"""
-        
-        id = u'Chap003'
+
+        identifier = u'Chap003'
         href = u'Text/chap3.xhtml'
         media_type = epub.MIMETYPE_OPF
-        
+
         manifest = epub.opf.Manifest()
-        item = epub.opf.ManifestItem(id, href, media_type)
-        
-        manifest.add_item(id, href, media_type)
-        
+        item = epub.opf.ManifestItem(identifier, href, media_type)
+
+        manifest.add_item(identifier, href, media_type)
+
         self.assertEqual(len(manifest), 1, u'Il manque un objet !')
-        self.assertIsInstance(manifest[id], epub.opf.ManifestItem)
+        self.assertIsInstance(manifest[identifier], epub.opf.ManifestItem)
 
     def test_as_xml_element(self):
         xml_string = u"""<manifest>
@@ -344,20 +534,17 @@ class TestManifest(unittest.TestCase):
         self.maxDiff = None
         doc = minidom.parseString(xml_string)
         xml_element = doc.documentElement
-        
+
         manifest = epub.opf._parse_xml_manifest(xml_element)
-        
+
         xml_input = xml_element.toxml().strip()
         xml_output = manifest.as_xml_element().toprettyxml(u'    ').strip()
-        
-        print xml_input
-        print xml_output
-        
+
         self.assertEqual(xml_output, xml_input)
 
 
 class TestGuide(unittest.TestCase):
-    
+
     def test_as_xml_element(self):
         xml_string = u"""<guide>
     <reference type="toc" title="Table of Contents" href="toc.html" />
@@ -368,15 +555,53 @@ class TestGuide(unittest.TestCase):
         self.maxDiff = None
         doc = minidom.parseString(xml_string)
         xml_element = doc.documentElement
-        
+
         guide = epub.opf._parse_xml_guide(xml_element)
-        
+
         self.assertEqual(guide.as_xml_element().toprettyxml(u'    ').strip(),
                          xml_element.toxml().strip())
 
 
 class TestSpine(unittest.TestCase):
-    
+
+    def test_init(self):
+        sp = epub.opf.Spine()
+        self.assertIsNone(sp.toc)
+        self.assertEquals(sp.itemrefs, [])
+
+        sp = epub.opf.Spine(u'ncx_file_id')
+        self.assertEquals(sp.toc, u'ncx_file_id')
+        self.assertEquals(sp.itemrefs, [])
+
+        itemrefs = [(u'text0001', True), (u'text0002', False)]
+        sp = epub.opf.Spine(u'ncx_file_id', itemrefs)
+        self.assertEquals(sp.toc, u'ncx_file_id')
+        self.assertEquals(sp.itemrefs, itemrefs)
+
+    def test_add_itemref(self):
+        sp = epub.opf.Spine()
+        sp.add_itemref(u'text0001')
+        self.assertEquals(sp.itemrefs, [(u'text0001', True)])
+
+        sp.add_itemref(u'text0002', True)
+        self.assertEquals(sp.itemrefs,
+                          [(u'text0001', True), (u'text0002', True)])
+
+        sp.add_itemref(u'text0003', False)
+        self.assertEquals(sp.itemrefs,
+                          [(u'text0001', True),
+                           (u'text0002', True),
+                           (u'text0003', False)])
+
+    def test_append(self):
+        sp = epub.opf.Spine()
+        sp.append((u'text0001', True))
+        self.assertEquals(sp.itemrefs, [(u'text0001', True)])
+
+        sp.append((u'text0002', False))
+        self.assertEquals(sp.itemrefs,
+                          [(u'text0001', True), (u'text0002', False)])
+
     def test_as_xml_element(self):
         xml_string = u"""<spine toc="ncx">
     <itemref idref="intro" />
@@ -392,18 +617,18 @@ class TestSpine(unittest.TestCase):
         self.maxDiff = None
         doc = minidom.parseString(xml_string)
         xml_element = doc.documentElement
-        
+
         spine = epub.opf._parse_xml_spine(xml_element)
-        
+
         self.assertEqual(spine.as_xml_element().toprettyxml(u'    ').strip(),
                          xml_element.toxml().strip())
 
 
 class TestOpf(unittest.TestCase):
-    
+
     def test_init(self):
         opf = epub.opf.Opf()
-        
+
         self.assertIsInstance(opf.metadata, epub.opf.Metadata)
         self.assertIsInstance(opf.manifest, epub.opf.Manifest)
         self.assertIsInstance(opf.spine, epub.opf.Spine)
@@ -413,18 +638,10 @@ class TestOpf(unittest.TestCase):
         xml_string = u"""<?xml version="1.0" ?>
 <package unique-identifier="BookId" version="2.0" xmlns="http://www.idpf.org/2007/opf">
     <metadata xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:opf="http://www.idpf.org/2007/opf">
-        <dc:title xml:lang="en">
-            Testing Epub
-        </dc:title>
-        <dc:creator opf:role="aut">
-            Florian Strzelecki
-        </dc:creator>
-        <dc:identifier id="BookId" opf:scheme="UUID">
-            urn:uuid:477d1a82-a70d-4ee5-a0ff-0dddc60fd2bb
-        </dc:identifier>
-        <dc:language>
-            en
-        </dc:language>
+        <dc:title xml:lang="en">Testing Epub</dc:title>
+        <dc:creator opf:role="aut">Florian Strzelecki</dc:creator>
+        <dc:identifier id="BookId" opf:scheme="UUID">urn:uuid:477d1a82-a70d-4ee5-a0ff-0dddc60fd2bb</dc:identifier>
+        <dc:language>en</dc:language>
         <meta content="0.4.2" name="Sigil version"/>
     </metadata>
     <manifest>
@@ -456,11 +673,8 @@ class TestOpf(unittest.TestCase):
 """
         self.maxDiff = None
         opf = epub.opf.parse_opf(xml_string)
-        
+
         xml_input = xml_string.strip()
         xml_output = opf.as_xml_document().toprettyxml(u'    ').strip()
-        
-        print xml_input
-        print xml_output
 
         self.assertEqual(xml_input, xml_output)
